# -*- coding: utf-8 -*-
"Tunir module to talk to Vagrant"

# Copyright © 2015-2016  Kushal Das <kushaldas@gmail.com>
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2, or (at your option) any later
# version.  This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.  You
# should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import os
import time
import logging
import subprocess
from .tunirutils import system
from typing import List, Dict, Set, Tuple, Union, Callable, TypeVar, Any, cast

log = logging.getLogger('tunir')


def refresh_vol_pool():
    '''Refreshes libvirt volume by removing extra files..

    '''
    out, err, retcode = system('virsh vol-list default')
    lines = out.split('\n')
    if len(lines) > 2:
        for line in lines[2:]:
            words = line.split()
            if len(words) == 2:
                if words[0].startswith('tunir-box'):
                    system('virsh vol-delete {0} default'.format(words[0]))

def refresh_storage_pool():
    '''Refreshes libvirt storage pool.

    http://kushaldas.in/posts/storage-volume-error-in-libvirt-with-vagrant.html
    '''
    out, err, retcode = system('virsh pool-list')
    lines = out.split('\n')
    if len(lines) > 2:
        for line in lines[2:]:
            words = line.split()
            if len(words) == 3:
                if words[1] == 'active':
                    system('virsh pool-refresh {0}'.format(words[0]))


def parse_ssh_config(text):
    # type: (str) -> Dict[str, str]
    """
    Parses the SSH config and returns a dict
    """
    result = {}
    lines = text.split('\n')
    if lines[0].strip().startswith('Host '):
        for line in lines[1:]:
            line = line.strip()
            words = line.split(' ')
            if len(words) == 2:
                result[words[0]] = words[1]

    return result


class Vagrant(object):
    """
    Returns a Vagrant object.
    """
    def __init__(self, image_url, name='tunir-box', memory=1024, provider='libvirt', path='/var/run/tunir/'):
        self.original_path = os.path.abspath(os.path.curdir)
        self.name = name
        self.image_url = image_url
        self.path = path
        self.keys = None # type: Dict[str, str]
        self.failed = False
        self.provider = provider

        libvirt_config = '''Vagrant.configure("2") do |config|
  config.vm.define :tunirserver do |tunirserver|
    tunirserver.vm.box = "{0}"
    tunirserver.vm.synced_folder ".", "/home/vagrant/sync", disabled: true
    tunirserver.vm.synced_folder ".", "/vagrant", disabled: true
    tunirserver.vm.provider :libvirt do |domain|
      domain.memory = {1}
      domain.cpus = 2
    end
  end
end'''

        virtualbox_config = '''Vagrant.configure("2") do |config|
  config.vm.box = "{0}"
  config.vm.synced_folder ".", "/home/vagrant/sync", disabled: true
  config.vm.synced_folder ".", "/vagrant", disabled: true
  config.vm.provider :virtualbox do |domain|
      domain.memory = {1}
      domain.cpus = 2
  end
end'''

        self.vagrantfile = None # type: str
        if self.provider == 'libvirt':
            self.vagrantfile = libvirt_config
            refresh_storage_pool()
        else:
            self.vagrantfile = virtualbox_config
        os.chdir(self.path)

        with open('Vagrantfile', 'w') as fobj:
            fobj.write(self.vagrantfile.format(name, memory))

        print "Wrote Vagrant config file."
        # Now actually register that image

        basename = os.path.basename(image_url)


        print "Adding vagrant box."
        cmd = 'vagrant box add {0} --name {1}'.format(image_url, name)
        log.info(cmd)
        out, err, retcode = system(cmd)
        # Now check for error, I will skip this.
        if retcode != 0:
            print("Error while trying to add the box.")
            print err
            self.failed = True
            return
        print out

        print "Up the vagrant"

        # Let us up the vagrant
        cmd = 'vagrant up --provider {0}'.format(self.provider)
        log.info(cmd)
        out, err, retcode = system(cmd)
        if retcode != 0:
            print("Error while trying to do vagrant up the box.")
            print err
            self.failed = True
            return
        print out

        time.sleep(3)
        print "Time to get Vagrant ssh-config"


        # Now let us try to get the ssh-config
        cmd = 'vagrant ssh-config'
        log.info(cmd)
        out, err, retcode = system(cmd)
        if retcode != 0:
            print("Error while trying to get ssh config for the box.")
            print err
            self.failed = True
            return
        print out
        self.keys = parse_ssh_config(out)
        os.chdir(self.original_path)

    def destroy(self):
        os.chdir(self.path)
        print "Let us destroy the box."
        cmd = 'vagrant destroy -f'
        log.info(cmd)
        out, err, retcode = system(cmd)
        if retcode != 0:
            print("Error while trying to destroy the instance.")
            print err

        cmd = 'vagrant box remove {0} -f'.format(self.name)
        log.info(cmd)
        out, err, retcode = system(cmd)
        if retcode != 0:
            print("Error while trying to remove the box.")
            print err

        if self.provider == 'libvirt':
            refresh_vol_pool() # Remove libvirt cache
        os.chdir(self.original_path)

def vagrant_and_run(config, path='/var/run/tunir/'):
    """
    This starts the vagrant box

    :param config: Our config object
    :return: (Vagrant, config) config object with IP, and key file
    """
    v = Vagrant(config['image'], memory=config['ram'],
                provider=config.get('provider', 'libvirt'), path=path)
    if v.keys: # Means we have the box up, and also the ssh config
        config['host_string'] = v.keys['HostName']
        config['ip'] = v.keys['HostName']
        config['key'] = v.keys['IdentityFile'].strip('"')
        config['port'] = v.keys['Port']

    return v, config




if __name__ == '__main__':
    data = '''Host default
  HostName 192.168.121.18
  User vagrant
  Port 22
  UserKnownHostsFile /dev/null
  StrictHostKeyChecking no
  PasswordAuthentication no
  IdentityFile /root/vg/.vagrant/machines/default/libvirt/private_key
  IdentitiesOnly yes
  LogLevel FATAL
'''
    print parse_ssh_config(data)




import os
import unittest
import sys
import tempfile
from collections import OrderedDict
from contextlib import contextmanager
from StringIO import StringIO
from mock import patch, Mock

import tunirlib
from tunirlib.tunirutils import Result, system
from tunirlib import main
from tunirlib import tunirutils, tunirmultihost, tunirvagrant

@contextmanager
def captured_output():
    new_out, new_err = StringIO(), StringIO()
    old_out, old_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = new_out, new_err
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout, sys.stderr = old_out, old_err

class StupidProcess(object):
    """
    For testing vm creation.
    """
    def __init__(self):
        self.pid = 42 # Answer to all problems.

class StupidArgs(object):
    """
    For testing vm creation.
    """
    def __init__(self):
        self.job = 'testvm'
        self.config_dir = './'
        self.atomic = None
        self.image_dir = None
        self.multi = None
        self.debug = False


class TunirTests(unittest.TestCase):
    """
    Tests tunir's codebase
    """

    def test_single_config(self):
        data = tunirlib.read_job_configuration(jobname="fedora", config_dir='./testvalues/')
        self.assertEqual(2048, data["ram"], "Missing ram information")


    def test_match_vm_numbers(self):
        path = './testvalues/multihost.txt'
        vms = ['vm1', 'vm2']
        self.assertTrue(tunirutils.match_vm_numbers(vms, path))
        vms = ['vm1',]
        with captured_output() as (out, err):
            self.assertFalse(tunirutils.match_vm_numbers(vms, path))
            self.assertIn('vm2', out.getvalue())

    def test_ansible(self):
        vms = {'vm1': {'ip': '192.168.1.100', 'user':'fedora'},\
               'vm2': {'ip': '192.168.1.102', 'user':'fedora'}}
        tdir = tempfile.mkdtemp()
        new_inventory = os.path.join(tdir, 'tunir_ansible')
        old_inventory = os.path.join(tdir, 'inventory')
        with open(old_inventory, 'w') as fobj:
            fobj.write('[web]\nvm1\nvm2')
        tunirutils.create_ansible_inventory(vms, new_inventory)
        self.assertTrue(os.path.exists(new_inventory))
        with open(new_inventory) as fobj:
            data = fobj.read()
        tunirutils.clean_tmp_dirs([tdir,])
        self.assertIn('vm2 ansible_ssh_host=192.168.1.102 ansible_ssh_user=fedora\n', data)
        self.assertIn('vm1 ansible_ssh_host=192.168.1.100 ansible_ssh_user=fedora\n', data)
        self.assertIn('[web]\nvm1\nvm2', data)


    @patch('tunirlib.tunirutils.run')
    @patch('codecs.open')
    @patch('subprocess.call')
    @patch('subprocess.Popen')
    @patch('os.system')
    @patch('time.sleep')
    @patch('sys.exit')
    @patch('os.kill')
    @patch('paramiko.SSHClient')
    @patch('tunirlib.tunirmultihost.boot_qcow2')
    def test_multihost(self,p_br,p_sc,p_kill, p_exit, p_sleep, p_system,p_usystem, p_scall, p_codecs,p_run):
        res = StupidProcess()
        p_br.side_effect = [(res, 'ABCD'),(res,'XYZ')]

        r1 = Result("result1")
        r1.return_code = 0
        r2 = Result("result2")
        r2.return_code = 0
        r3 = Result("result3")
        r3.return_code = 0
        values = [r1, r2, r3]
        p_run.side_effect = values

        c1 = Mock()
        c1.communicate.return_value = ('192.168.1.100','')
        c1.return_code = 0
        c2 = Mock()
        c2.communicate.return_value = ('? (192.168.122.100) at ABCD [ether] on virbr0\n? (192.168.122.116) at 00:16:3e:33:ba:a2 [ether] on virbr0','')
        c2.return_code = 0
        c3 = Mock()
        c3.communicate.return_value = ('h','h')
        c3.return_code = 0
        c4 = Mock()
        c4.communicate.return_value = ('? (192.168.122.102) at XYZ [ether] on virbr0\n? (192.168.122.116) at 00:16:3e:33:ba:a2 [ether] on virbr0','')
        c4.return_code = 0
        p_usystem.side_effect = [c1,c2,c3,c4]


        with captured_output() as (out, err):
            tunirmultihost.start_multihost('multihost', './testvalues/multihost.txt', debug=False, config_dir='./testvalues/')

            #self.assertIn("Passed:1", out.getvalue())
            #self.assertIn("Job status: True", out.getvalue())
        for calls,vals in zip(p_run.mock_calls,(('192.168.122.100', 22, 'fedora', None, 'sudo su -c"echo Hello > /abcd.txt"'),('192.168.122.102', 22, 'fedora', None, 'ls /'),('192.168.122.100', 22, 'fedora', None, 'cat /etc/os-release'))):
            self.assertEqual(calls[1],vals)


class ExecuteTests(unittest.TestCase):
    """
    Tests the execute function.
    """

    @patch('tunirlib.tunirutils.run')
    def test_execute(self, t_run):
        config = {"host_string": "127.0.0.1",
                  "user": "fedora"}
        r1 = Result("result1")
        r1.return_code = 0
        t_run.return_value=r1
        res, negative = tunirutils.execute(config, 'ls')
        self.assertEqual(str(res), "result1")
        self.assertEqual(negative, "no")

    @patch('tunirlib.tunirutils.run')
    def test_execute_nagative(self, t_run):
        config = {"host_string": "127.0.0.1",
                  "user": "fedora"}
        r1 = Result("result1")
        r1.return_code = 127
        t_run.return_value=r1
        res, negative = tunirutils.execute(config, '@@ ls')
        self.assertEqual(str(res), "result1")
        self.assertEqual(negative, "yes")

    @patch('tunirlib.tunirutils.run')
    def test_execute_nongating(self, t_run):
        config = {"host_string": "127.0.0.1",
                  "user": "fedora"}
        r1 = Result("result1")
        r1.return_code = 127
        t_run.return_value=r1
        res, negative = tunirutils.execute(config, '## ls')
        self.assertEqual(str(res), "result1")
        self.assertEqual(negative, "dontcare")

class UpdateResultTest(unittest.TestCase):
    """
    Tests the update_result function.
    """
    def setUp(self):
        tunirutils.STR = OrderedDict()

    def test_updateresult(self):

        r1 = Result("result1")
        r1.return_code = 0
        r2 = Result("result2")
        r2.return_code = 127
        r3 = Result("result3")
        r3.return_code = 1
        values = [(r1, 'no', 'ls'), (r2, 'yes', '@@ sudo reboot'), (r3, 'dontcare', '## ping foo')]
        for res, negative, command in values:
            tunirutils.update_result(res, command, negative)

        res = [True, True, False]
        for out, result in zip(tunirlib.STR.iteritems(), res):
            self.assertEqual(out[1]['status'], result)


class TestVagrant(unittest.TestCase):
    "To test the vagrant class"

    @patch('tunirlib.tunirvagrant.system')
    def test_refresh_vol_pool(self, t_sys):
        line = """ Name                 Path
------------------------------------------------------------------------------
 tunir-box.qcow2        /var/lib/libvirt/images/tunir-box.qcow2 """
        t_sys.return_value = (line,"no error",0)
        tunirvagrant.refresh_storage_pool()
        self.assertTrue(t_sys.called)
        t_sys.assert_called_with('virsh pool-list')


if __name__ == '__main__':
    unittest.main()




import os
import sys
import json
import argparse
from tunirvagrant import vagrant_and_run
from tuniraws import aws_and_run
from tunirmultihost import start_multihost
from tunirutils import run_job, Result
from collections import OrderedDict
import logging
from systemd.journal import JournalHandler

log = logging.getLogger('tunir')
log.addHandler(JournalHandler(SYSLOG_IDENTIFIER='tunir'))
log.setLevel(logging.DEBUG)


STR = OrderedDict() # type: Dict[str, Dict[str, str]]


def read_job_configuration(jobname='', config_dir='./'):
    """
    :param jobname: Name of the job
    :param config_dir: Directory for configuration.
    :return: Configuration dict
    """
    data = None
    name = jobname + '.json'
    name = os.path.join(config_dir, name)
    log.info("Reading configuration {0}".format(name))
    if not os.path.exists(name):
        print "Job configuration is missing."
        return None
    with open(name) as fobj:
        data = json.load(fobj)
    return data

def main(args):
    "Starting point of the code"
    job_name = ''
    node = None
    debug = False
    return_code = -100
    run_job_flag = True

    if args.debug:
        debug = True
    # For multihost
    if args.multi:
        jobpath = os.path.join(args.config_dir, args.multi + '.txt')
        status = start_multihost(args.multi, jobpath, debug, config_dir=args.config_dir)
        os.system('stty sane')
        if status:
            sys.exit(0)
        else:
            sys.exit(2)
    if args.job:
        job_name = args.job
    else:
        sys.exit(-2)

    jobpath = os.path.join(args.config_dir, job_name + '.txt')


    # First let us read the vm configuration.
    config = read_job_configuration(job_name, args.config_dir)
    if not config: # Bad config name
        sys.exit(-1)

    os.system('mkdir -p /var/run/tunir')
    if config['type'] == 'vm':
        status = start_multihost(job_name, jobpath, debug, config, args.config_dir)
        if status:
            return_code = 0
        os.system('stty sane')
        sys.exit(return_code)

    if config['type'] == 'vagrant':
        node, config = vagrant_and_run(config)
        if node.failed:
            run_job_flag = False

    elif config['type'] == 'aws':
        node, config = aws_and_run(config)
        if node.failed:
            run_job_flag = False
        else:
            print "We have an instance ready in AWS.", node.node

    elif config['type'] == 'bare':
        config['host_string'] = config['image']
        config['ip'] = config['image']
        config['port'] = config.get('port', '22')
        run_job_flag = True
    try:
        if run_job_flag:
            status = start_multihost(job_name, jobpath, debug, config, args.config_dir)
            if status:
                return_code = 0
    finally:
        if config['type'] != 'bare':
            # Destroy and remove the Vagrant box or AWS instance
            node.destroy()

        sys.exit(return_code)


def startpoint():
    parser = argparse.ArgumentParser()
    parser.add_argument("--job", help="The job configuration name to run for JSON format file.")
    parser.add_argument("--config-dir", help="Path to the directory where the job config and commands can be found.",
                        default='./')
    parser.add_argument("--debug", help="Keep the vms running for debug in multihost mode.", action='store_true')
    parser.add_argument("--multi", help="The multivm configuration using .cfg configuration file")
    args = parser.parse_args()

    main(args)

if __name__ == '__main__':
    startpoint()



# -*- coding: utf-8 -*-
# Copyright 2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

"""
This is a module for downloading fedora cloud images (and probably any other
qcow2) and then booting them locally with qemu.
"""

import subprocess
import config


def create_user_data(path, password):
    # type: (str, str) -> str
    "Creates a simple user data file"
    file_data = config.USER_DATA % password
    with open(path + '/meta/user-data', 'w') as user_file:
        user_file.write(file_data)
    return "user-data file generated."


def create_seed_img(meta_path, img_path):
    # type: (str, str) -> str
    """Create a virtual filesystem needed for boot with virt-make-fs on a given
    path (it should probably be somewhere in '/tmp'."""

    make_image = subprocess.call(['virt-make-fs',
                                  '--type=msdos',
                                  '--label=cidata',
                                  meta_path,
                                  img_path + '/seed.img'])

    if make_image == 0:
        return "seed.img created at %s" % img_path

    return "creation of the seed.img failed."





import os
import sys
import time
import signal
import random
import cStringIO
from Crypto.PublicKey import RSA
from paramiko.rsakey import RSAKey
import subprocess
import tempfile
import ConfigParser
import logging
from pprint import pprint
from .tunirutils import run, clean_tmp_dirs, system, run_job
from .tunirutils import match_vm_numbers, create_ansible_inventory
from .tunirutils import IPException
from .testvm import  create_user_data, create_seed_img
log = logging.getLogger('tunir')

def true_test(vms, private_key, command='cat /proc/cpuinfo'):
    """
    Runs a given command to the list of vms. Currently using it
    to push the list of vms/ips to the /etc/hosts files.

    :param vms: Dictionary of the VM(s) with ip addresses to work on
    :param private_key: String version of the private key to ssh
    :param command: The actual command to run.
    :return: None
    """
    "Just to test the connection of a vm"
    key = create_rsa_key(private_key)
    for vm in vms.values():
        for i in range(5):
            try:
                res = run(vm['ip'], port=vm['port'], user=vm['user'], command=command,pkey=key, debug=False)
                break
            except Exception as e:
                print("Try {0} failed for IP injection to /etc/hosts.".format(i))
                if i == 4: # If it does not allow in 2 minutes, something is super wrong
                    raise e
                time.sleep(30)
                continue

def inject_ip_to_vms(vms, private_key):
    """
    Updates each vm's /etc/hosts file with IP addresses.

    :param vms: Dictionary of VM(s)/IPs
    :param private_key: String version of the private key
    :return: None
    """

    text = "\n"
    for k, v in vms.iteritems():
        line = ''
        # ip hostname format for /etc/hosts
        if 'hostname' in v:
            line = "{0}    {1} {2}\n".format(v['ip'],k,v['hostname'])
        else:
            line = "{0}    {1}\n".format(v['ip'],k)
        text += line
    true_test(vms, private_key, """sudo sh -c 'echo -e "{0}" >> /etc/hosts'""".format(text))

def create_rsa_key(private_key):
    """ Creates the RSAKey for paramiko.
    :param private_key: String version of the private key
    :return: The RSAKey object to be used in paramiko
    """
    fobj = cStringIO.StringIO(private_key)
    key = RSAKey.from_private_key(file_obj=fobj)
    return key

def generate_sshkey(bits=2048):
    '''
    Returns private key and public key, and the key object
    '''
    key = RSA.generate(bits, e=65537)
    public_key = key.publickey().exportKey("OpenSSH")
    private_key = key.exportKey("PEM")
    return private_key, public_key, key

def scan_arp(macaddr):
    "Find the ip for the given mac addr"
    output, err, eid = system('arp -an')
    lines = output.split('\n')
    for line in lines:
        words = line.split(' ')
        if len(words) > 3:
            if words[3].strip() == macaddr:
                return words[1].strip('()')

def read_multihost_config(filepath):
    '''Reads the given filepath, and returns a dict with all required information.
    '''
    result = {}
    config = ConfigParser.RawConfigParser()
    config.read(filepath)
    sections = config.sections()
    for sec in sections:
        items = config.items(sec)
        out = dict(items)
        result[sec] = out
    return result

def random_mac():
    """
    Generates a random MAC address
    :return: A string containing the random MAC address.
    """
    mac = [ 0x00, 0x16, 0x3e,\
        random.randint(0x00, 0x7f),\
        random.randint(0x00, 0xff),\
        random.randint(0x00, 0xff) ]
    return ':'.join(map(lambda x: "%02x" % x, mac))

def boot_qcow2(image, seed, ram=1024, vcpu='1'):
    "Boots the image with a seed image"
    mac = random_mac()
    boot_args = ['/usr/bin/qemu-kvm',
                 '-m',
                 str(ram),
                 '-smp',
                 vcpu,
                 '-drive',
                 'file=%s,if=virtio' % image,
                 '-drive',
                 'file=%s,if=virtio' % seed,
                 '-net',
                 'bridge,br=virbr0',
                 '-net',
                 'nic,macaddr={0},model=virtio'.format(mac),
                 '-display',
                 'none'
                 ]
    print(' '.join(boot_args))
    vm = subprocess.Popen(boot_args)

    print("Successfully booted your local cloud image!")

    return vm, mac

def create_ssh_metadata(path, pub_key, private_key=None):
    "Creates the user data with ssh key"
    text = """instance-id: iid-123456
local-hostname: tunirtests
public-keys:
  default: {0}
"""
    fname = os.path.join(path, 'meta/meta-data')
    with open(fname, 'w') as fobj:
        fobj.write(text.format(pub_key))

    # just for testing
    if private_key:
        pname = os.path.join(path, 'private.pem')
        with open(pname,'w') as fobj:
            fobj.write(private_key)
        os.system('chmod 0600 {0}'.format(pname))

def start_multihost(jobname, jobpath, debug=False, oldconfig=None, config_dir='.'):
    "Start the executation here."
    temppath = tempfile.mktemp()
    extra_config = {'result_path' : temppath}
    print('Result file at: {0}'.format(temppath))
    status = 0
    ansible_inventory_path = None
    fault_in_ip_addr = False
    private_key = None
    config_path = os.path.join(config_dir, jobname + '.cfg')
    if debug:
        print(config_path)
    config = None
    vcpu = '1'
    vms = {} # Empty directory to store vm details
    dirs_to_delete = [] # We will delete those at the end
    vm_keys = None
    if not oldconfig:
        config = read_multihost_config(config_path)
        ram = config.get('general').get('ram', '1024')
        vcpu = config.get('general').get('cpu', '1')
        result_path = config.get('general').get('result_path', None)
        if result_path:
            extra_config['result_path'] = result_path
        vm_keys = [name for name in config.keys() if name.startswith('vm')]
        if 'key' in config['general']:
            data = ''
            try:
                with open(config['general']['key']) as fobj:
                    data = fobj.read()
            except Exception as e:
                print(e)
                raise e
            config['general']['pkey'] = create_rsa_key(data)
            private_key = data
            config['general']['keypath'] = config['general']['key']
    else: # For a single vm job or Vagrant or AWS
        config = {'vm1': oldconfig}
        config['general'] = {'ansible_dir': oldconfig.get('ansible_dir', None)}
        if 'key' in oldconfig:
            data = ''
            with open(oldconfig['key']) as fobj:
                data = fobj.read()
            config['general']['pkey'] = create_rsa_key(data)
            config['general']['keypath'] = oldconfig['key']
            private_key = data
            config['general']['key'] = data
        ram = oldconfig.get('ram', '1024')
        vm_keys = ['vm1',]
    #TODO Parse the job file first
    if not os.path.exists(jobpath):
        print("Missing job file {0}".format(jobpath))
        return False

    # For extra vm(s) in the job file fail fast
    if not match_vm_numbers(vm_keys, jobpath):
        return

    # First let us create the seed image
    seed_dir = tempfile.mkdtemp()
    print('Created {0}'.format(seed_dir))
    os.system('chmod 0777 %s' % seed_dir)
    dirs_to_delete.append(seed_dir)
    if 'key' not in config['general']:
        # Then we create key and metadata
        meta = os.path.join(seed_dir, 'meta')
        os.makedirs(meta)
        print("Generating SSH keys")
        private_key, public_key, KEY = generate_sshkey()
        create_user_data(seed_dir, "passw0rd")
        create_ssh_metadata(seed_dir, public_key, private_key)
        create_seed_img(meta, seed_dir)
        seed_image = os.path.join(seed_dir, 'seed.img')

        # We will copy the seed in every vm run dir
        pkey = create_rsa_key(private_key)

    try:
        for vm_c in vm_keys:
            # Now create each vm one by one.
            # Get the current ips
            this_vm = {}
            if 'ip' not in config[vm_c]:
                current_d = tempfile.mkdtemp()
                print('Created {0}'.format(current_d))
                os.system('chmod 0777 %s' % current_d)
                dirs_to_delete.append(current_d)
                system('cp  {0} {1}'.format(seed_image, current_d))
                # Next copy the qcow2 image
                image_path = config[vm_c].get('image')
                os.system('cp {0} {1}'.format(image_path, current_d))
                image = os.path.join(current_d, os.path.basename(image_path))
                log.info("Booting {0}".format(image))

                vm, mac = boot_qcow2(image, os.path.join(current_d, 'seed.img'), ram, vcpu=vcpu)
                this_vm.update({'process': vm, 'mac': mac})
                # Let us get this vm in the tobe delete list even if the IP never comes up
                vms[vm_c] = this_vm
                print("We will wait for 45 seconds for the image to boot up.")
                time.sleep(45)
                latest_ip = scan_arp(mac)
                if not latest_ip:
                    fault_in_ip_addr = True
                    break
                this_vm['ip'] = latest_ip
                this_vm['host_string'] = latest_ip
                this_vm['pkey'] = pkey
                this_vm['port'] = config[vm_c].get('port', 22)
            else:
                this_vm['ip'] = config[vm_c].get('ip')
                this_vm['host_string'] = config[vm_c].get('ip')
                this_vm['port'] = config[vm_c].get('port', 22)
                this_vm['pkey'] = config['general']['pkey']

            this_vm['user'] = config[vm_c].get('user')
            if 'hostname' in config[vm_c]:
                this_vm['hostname'] = config[vm_c].get('hostname')
            log.info("IP of the new instance: {0}".format(this_vm.get('ip')))

            vms[vm_c] = this_vm
        only_vms = vms.copy()
        vms['general'] = config['general']
        if fault_in_ip_addr:
            print('Oops no IP for this vm.')
            raise IPException

        # Now we are supposed to have all the vms booted.
        if debug:
            pprint(vms)
        print(' ')
        inject_ip_to_vms(only_vms, private_key)
        ansible_flag = config.get('general').get('ansible_dir', None)
        if ansible_flag:
            dir_to_copy = ansible_flag
            if not dir_to_copy.endswith('/'):
                dir_to_copy += '/*'
            else:
                dir_to_copy += '*'
            os.system('cp -r {0} {1}'.format(dir_to_copy, seed_dir))
            ansible_inventory_path = os.path.join(seed_dir, 'tunir_ansible')
            create_ansible_inventory(only_vms, ansible_inventory_path)


        # This is where we test

        status = run_job(jobpath,job_name=jobname,vms=vms, ansible_path=seed_dir, extra_config=extra_config)
    except Exception as e:
        import traceback
        exc_type, exc_value, exc_traceback = sys.exc_info()
        print "*** print_tb:"
        traceback.print_tb(exc_traceback, limit=1, file=sys.stdout)
        print "*** print_exception:"
        traceback.print_exception(exc_type, exc_value, exc_traceback,
                              limit=2, file=sys.stdout)

    finally:
        if debug:
            filename = os.path.join(seed_dir, 'destroy.sh')
            with open(filename, 'w') as fobj:
                for vm in only_vms.values():
                    if not 'process' in vm: # For remote vm/bare metal
                        continue
                    job_pid = vm['process'].pid
                    fobj.write('kill -9 {0}\n'.format(job_pid))
                for d in dirs_to_delete:
                    fobj.write('rm -rf {0}\n'.format(d))
            print("DEBUG MODE ON. Destroy from {0}".format(filename))
            # Put the ip/hostnames into a text file
            filename = os.path.join(seed_dir, 'hostnames.txt')
            with open(filename, 'w') as fobj:
                for k, v in only_vms.iteritems():
                    fobj.write('{0}={1}\n'.format(k,v['ip']))
            return status # Do not destroy for debug case
        for vm in only_vms.values():
            if not 'process' in vm: # For remote vm/bare metal
                continue
            job_pid = vm['process'].pid
            if debug:
                print('Killing {0}'.format(job_pid))
            os.kill(job_pid, signal.SIGKILL)
        clean_tmp_dirs(dirs_to_delete)
        return status








